package interfaces;

import tools.HasInvariant;

/**
 * When an elevator has the security option as specified in {@link enumerations.SecurityType} then 
 * certain floors can be locked and unlocked so that the buttons within the elevator are disbaled and enabled
 * @author jpaulgibson
 *
 */
public interface ElevatorWithLockableFloorsSpecification extends HasInvariant{
	
	/**
	 * @param floor to be locked
	 */
	public void lock (int floor);
	
	/**
	 * @param floor to be unlocked
	 */
	public void unlock (int floor);
	
	
	/**
	 * @param floor to be checked
	 * @return if it is lockable
	 */
	public boolean is_lockable(int floor);
	
	/**
	 * @param floor to be checked
	 * @return if it is locked
	 */
	public boolean is_locked(int floor);
	
	/**
	 * Defines when floor locks are in a safe state
	 * @return if all floors that are locked are also lockable
	 */
	@Override
	public boolean invariant();
}
