package models.elevatorLogicUnit;

import enumerations.LogicType;
import enumerations.DirectionState;
import abstractions.LogcUnitForElevatorBinding;
import interfaces.ElevatorForLogicUnitSpecification;

/**
 * The behaviour of an elevator when there is an emergency to be managed:<br>
 * the elevator must always stay at the current floor and stop at the next floor if already moving.<br>
 * This corresponds to EMERGENCY in the {@link enumerations.LogicType}
 * Tested by {@link unit_tests.controllersSpecifyingLogicForMoving.JUnit_EmergencyController}
 * @author J Paul Gibson
 */
public class EmergencyLogicUnit extends LogcUnitForElevatorBinding{

	
	public EmergencyLogicUnit(ElevatorForLogicUnitSpecification elevator) {
		
	       super(elevator);	     
	}
	
    /**
     * In an emergency situation the controller must always stay at the current floor<br>
     * Tested by {@link unit_tests.controllersSpecifyingLogicForMoving.JUnit_EmergencyController#test_calculateDirection()}
     */
	public DirectionState calculateDirection(){	
		return DirectionState.STAY;
	}
	
	/**
	 * In an emergency situation the controller must stop at the next floor<br>
	 * Tested by {@link unit_tests.controllersSpecifyingLogicForMoving.JUnit_EmergencyController#test_stopAtNextFloor()}
	 */
	public boolean stopAtNextFloor(){
		
		return true;
	}
	
	public String toString (){
		
		return LogicType.EMERGENCY.toString();
	}

}
