package models.elevatorLogicUnit;

import java.util.Random;

import enumerations.LogicType;
import enumerations.DirectionState;
import abstractions.LogcUnitForElevatorBinding;
import interfaces.ElevatorForLogicUnitSpecification;

/**
 * The random  controller behaves randomly as follows:
 * <ul>
 * <li> At each floor there is a 50-50 chance that it will stop </li>
 * <li> When stopped, the direction of the next movement is chosen randomly (with equal chance for all possible values)</li>
 * </ul>
 * This corresponds to RANDOM in the {@link enumerations.LogicType}
 * @author J Paul Gibson
 */
public class RandomMovementLogicUnit extends LogcUnitForElevatorBinding{
	
	Random RNG;
	
	public RandomMovementLogicUnit (ElevatorForLogicUnitSpecification elevator) {
		
       super(elevator);	
	   RNG = new Random();
	}
	
	public RandomMovementLogicUnit (ElevatorForLogicUnitSpecification elevator, Random RNG) {
		
		super(elevator);	
		
		if (RNG == null) this.RNG = new Random(); 
		            else this.RNG = RNG;
	}
	

    /**
     * The new direction is chosen randomly (with equal chance for all possible values)<br>
     */
	public DirectionState calculateDirection() {
		
		// When on bottom floor, choose to go UP or to STAY
		if (elevator.currentFloor()==0){
			int choice = RNG.nextInt(2);
			if (choice == 0) return DirectionState.UP;
			else return DirectionState.STAY;
			
		}
		
		// When on top floor, choose to go DOWN or to STAY
		if (elevator.currentFloor()==elevator.topFloor()){
			int choice = RNG.nextInt(2);
			if (choice == 0) return DirectionState.DOWN;
			else return DirectionState.STAY;
			
		}
		
		// When on floor with floors above and below, choose to go UP, DOWN or to STAY
		int choice = RNG.nextInt(3);
		if (choice == 0) return DirectionState.UP;
		else if (choice == 1) return DirectionState.DOWN;
		else return DirectionState.STAY;
		
	}

    /**
     * At each floor there is a 50-50 chance that it will stop 
     */
	public boolean stopAtNextFloor(){
		
		int choice = RNG.nextInt(2);
		if (choice == 0) return true;
		return false;
	}

	
	public String toString (){
		
		return LogicType.RANDOM.toString();
	}

}
