package models.elevatorLogicUnit;

import enumerations.LogicType;
import enumerations.DirectionState;
import abstractions.LogcUnitForElevatorBinding;
import interfaces.ElevatorForLogicUnitSpecification;

/**
 * The stupid controller ensures that the elevator moves from bottom to top to bottom ... 
 * stopping at  every floor.
 * This corresponds to STUPID in the {@link enumerations.LogicType}
 * @author J Paul Gibson
 */
public class StupidLogicUnit extends LogcUnitForElevatorBinding{

	
	public StupidLogicUnit (ElevatorForLogicUnitSpecification elevator) throws IllegalArgumentException{
		
		super(elevator);
	}
	

    /**
     * If the elevator is on the bottom floor then set the direction to up<br>
     * If the elevator is on the top floor then set the direction to down<br>
     * Else If the elevator has been stopped then set off towards bottom<br>
     * Otherwise keep moving in same direction
     */
	public DirectionState calculateDirection() {
		
		if (elevator.currentFloor()==0) return DirectionState.UP;
		if (elevator.currentFloor()== elevator.topFloor()) return DirectionState.DOWN;
		if (elevator.currentDirection() == DirectionState.STAY) return DirectionState.DOWN;
		else return elevator.currentDirection();
	}

	/**
	 * Always stop at the next floor
	 */
	public boolean stopAtNextFloor(){
		return true;
	}
	
	public String toString (){
			
			return LogicType.STUPID.toString();
	}

}
