 package models.elevatorUsers;

import interfaces.ElevatorForUsersSpecification;
import enumerations.DirectionState;
import enumerations.UserState;

import java.util.Random;

import abstractions.ElevatorUser;


/**
 * Tested by:
 * <ul> 
 * <li> unit tests -  {@link unit_tests.elevatorUsers.JUnit_IntelligentElevatorUser} </li>
 * <li> validation tests -  {@link validation_tests.Validation_IntelligentElevatorUser} </li>
 * </ul>
 * This elevator user behaves intelligently as defined in {@link #makeDecision()}<br>
 * This specialisation corresponds to INTELLIGENT in the {@link enumerations.UserType}
 * 
 * @author jpaulgibson
 */
public class IntelligentElevatorUser extends ElevatorUser {
	

	public IntelligentElevatorUser (int startFloor, int endFloor, ElevatorForUsersSpecification elevator) throws IllegalArgumentException, IllegalStateException{
		
		super(startFloor, endFloor, elevator);	
	}
	
	public IntelligentElevatorUser (Random RNG, ElevatorForUsersSpecification elevator) throws IllegalArgumentException, IllegalStateException{
		
		super(RNG, elevator);		
	}

	
	public boolean equals (Object obj){
		
		if (obj == null) return false;
		if (this == obj) return true;
		if (! (obj instanceof IntelligentElevatorUser)) return false;
		IntelligentElevatorUser u = (IntelligentElevatorUser) obj;
		return this.unique_ID == u.unique_ID;
		
	}
	
	public String toString (){
		String str = "IntelligentElevatorUser "+ unique_ID+ " - ";
		str = str + "from: " + startFloor;
		str = str + " to: "+  endFloor;
		str = str + " currently "+ state;
		return str;
	}
	
	/**
	 * This Elevator User Behaves Intelligently:
	 * <ul>
	 * <li> Press the correct direction button when they request the elevator at the floor </li>
	 * <li> Enter the elevator when it stops at their floor and is travelling in the required direction </li>
	 * <li> Presses the correct floor destination button inside the elevator </li>
	 * <li> Leaves the elevator when it arrives at the desired destination floor </li>
	 * </ul>
	 */
	@Override
	public void makeDecision() {
		
		// What direction does the user wish to travel?
		DirectionState user_direction;
		 if (startFloor < endFloor) user_direction = DirectionState.UP;
		 else user_direction = DirectionState.DOWN;
		
		// Which floor button(s) does the user press when they arrive at the floor?
		if (state == UserState.ARRIVED_AT_FLOOR){
		if (user_direction == DirectionState.UP)
		     elevator.pressUp(startFloor);
		 else  
			 elevator.pressDown(startFloor);
		 state = UserState.WAITING_AT_FLOOR;
		}
		
		// Does the user get into the elevator and what buttons do they press?
		else if (startFloor == elevator.currentFloor() && 
    		 state == UserState.WAITING_AT_FLOOR &&
    		 user_direction == elevator.currentDirection() 
    		 ){
    			 pickedUpAtFloor();
    			 elevator.pressFloorInElevator(endFloor);
    		 }
		 
		 // Does the user get out of the elevator?
		else if (endFloor== elevator.currentFloor() &&
    		 state  == UserState.IN_ELEVATOR) deliveredToFloor();
	}			

}
	

