package unit_tests.doors;

import models.elevatorDoors.DoorsOnFloors;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

/**
 * Unit validation_tests for {@link DoorsOnFloors} <br>
 * @author J Paul Gibson
 * @see validation_tests.Validation_DoorsOnFloors
 */
public class JUnit_DoorsOnFloors {

private final int NUMBER_OF_FLOORS = 10;
	
	DoorsOnFloors doorsUnderTest;
	
	@Before
	public  void setUp() throws Exception{
		
		doorsUnderTest = new DoorsOnFloors(NUMBER_OF_FLOORS);
	}
	
	@After
	public void tearDown() throws Exception {
		doorsUnderTest = null;
	}

	/**
	 * Unit test for {@link DoorsOnFloors#invariant()}<br>
	 * Tests that the constructor initialises the buttons in a safe state
	 */
	@Test
	public void test_invariant(){
		
		Assert.assertTrue(doorsUnderTest.invariant());
	}
	
	/**
	 * Unit test for {@link DoorsOnFloors#isDoorOpen(int )}
	 */
	@Test
	public void test_isDoorOpen(){
		
		for (int i=0; i< NUMBER_OF_FLOORS;i++) 
			Assert.assertFalse(doorsUnderTest.isDoorOpen(i));
		
		Assert.assertTrue(doorsUnderTest.invariant());
	}
	
	/**
	 * Unit test for {@link DoorsOnFloors#isDoorOpen(int ) exception case 1}
	 */
	@Test(expected=IllegalArgumentException.class)
	public void test_isDoorOpenException1(){
		
		doorsUnderTest.isDoorOpen(-1);
	}
	
	
	/**
	 * Unit test for {@link DoorsOnFloors#isDoorOpen(int ) exception case 2}
	 */
	@Test(expected=IllegalArgumentException.class)
	public void test_isDoorOpenException2(){
		
		doorsUnderTest.isDoorOpen(NUMBER_OF_FLOORS);
	}
	
	
	/**
	 * Unit test for {@link DoorsOnFloors#openDoor(int )}
	 */
	@Test
	public void test_openDoor(){
		
		doorsUnderTest.openDoor(0);
		
		Assert.assertTrue(doorsUnderTest.isDoorOpen(0));
		
		for (int i=1; i< NUMBER_OF_FLOORS;i++) 
			Assert.assertFalse(doorsUnderTest.isDoorOpen(i));
		
		Assert.assertTrue(doorsUnderTest.invariant());
	}
	
	
	/**
	 * Unit test for {@link DoorsOnFloors#openDoor(int ) exception case 1}
	 */
	@Test(expected=IllegalArgumentException.class)
	public void test_openDoorException1(){
		
		doorsUnderTest.openDoor(-1);
	
	}
	
	
	/**
	 * Unit test for {@link DoorsOnFloors#openDoor(int ) exception case 2}
	 */
	@Test(expected=IllegalArgumentException.class)
	public void test_openDoorException2(){
		
		doorsUnderTest.openDoor(NUMBER_OF_FLOORS);
	
	}
	
	/**
	 * Unit test for {@link DoorsOnFloors#openDoor(int ) exception case 3}
	 */
	@Test(expected=IllegalStateException.class)
	public void test_openDoorException3(){
		
		doorsUnderTest.openDoor(0);
		doorsUnderTest.openDoor(1);
	
	}
	
	
	/**
	 * Unit test for {@link DoorsOnFloors#closeDoor(int )}
	 */
	@Test
	public void test_closeDoor(){

		for (int i=1; i< NUMBER_OF_FLOORS;i++) {
			doorsUnderTest.openDoor(i);
			Assert.assertTrue(doorsUnderTest.isDoorOpen(i));
			Assert.assertTrue(doorsUnderTest.invariant());
			doorsUnderTest.closeDoor(i);
			Assert.assertFalse(doorsUnderTest.isDoorOpen(i));
			Assert.assertTrue(doorsUnderTest.invariant());
		}	
		
	}
	
	/**
	 * Unit test for {@link DoorsOnFloors#closeDoor(int ) exception case 1}
	 */
	@Test(expected=IllegalArgumentException.class)
	public void test_closeDoorException1(){
		
		doorsUnderTest.closeDoor(-1);
	
	}
	
	
	/**
	 * Unit test for {@link DoorsOnFloors#closeDoor(int ) exception case 2}
	 */
	@Test(expected=IllegalArgumentException.class)
	public void test_closeDoorException2(){
		
		doorsUnderTest.closeDoor(NUMBER_OF_FLOORS);
	
	}
	
	/**
	 * Unit test for {@link DoorsOnFloors#toString()}
	 */
	@Test
	public void test_toString(){
		
		Assert.assertEquals("DoorState States For Elevator At Floor: 0- 1- 2- 3- 4- 5- 6- 7- 8- 9-", 
				            doorsUnderTest.toString());
		
		doorsUnderTest.openDoor(2);
		
		Assert.assertEquals("DoorState States For Elevator At Floor: 0- 1- 2-OPEN 3- 4- 5- 6- 7- 8- 9-", 
	            doorsUnderTest.toString());
		
		doorsUnderTest.closeDoor(2);
        doorsUnderTest.openDoor(NUMBER_OF_FLOORS-1);
		
		Assert.assertEquals("DoorState States For Elevator At Floor: 0- 1- 2- 3- 4- 5- 6- 7- 8- 9-OPEN", 
	            doorsUnderTest.toString());
		
	}
	
}

