package unit_tests.elevators;

import org.junit.Assert;
import org.junit.Test;

import enumerations.DirectionState;
import models.elevatorLogicUnit.SmartLogicUnit;
import models.elevators.Elevator;
import unit_tests.elevators.unitTestAbstractions_elevators.JUnit_ElevatorSpecification;
import validation_tests.use_case10.Validation_Elevator_UseCase10;

/**
 * Simple unit test that runs use case scenario 1 as defined in {@link Validation_Elevator_UseCase10}.<br>
 * It checks only that the final state of the elevator is as expected.<br>
 * The same test can be validated in detail by executing {@link validation_tests.use_case10.Validation_Elevator_Smart}.
 * @author jpaulgibson
 */
public  class JUnit_Elevator_Smart extends JUnit_ElevatorSpecification{

	@Override
	public void setUp() throws Exception {
		        
		        // Make a smart controller, not yet bound to (installed in) a particular elevator
				SmartLogicUnit controller = new SmartLogicUnit(null);
				
				//Make a basic elevator, not yet bound to a particular controller
				elevatorUnderTest = new Elevator(Validation_Elevator_UseCase10.MINIMUM_NUMBER_OF_FLOORS);

				// Install the controller to be tested in the elevator
				elevatorUnderTest.installLogicUnit(controller);
		
	}
	
	
	/**        
     * Expected state after running use case scenario 1:
         Elevator State:
         DirectionState = DOWN
         Stopping at floor = true
         Floor = 2
         Number of floors = 5
         Requested Floors Inside Elevator:
         DirectionState Requests For Elevator At Floor: 0- 1- 2- 3- 4- 5-
         Controller type = Smart Controller 
	 */
	@Test
	public void test_useCaseScenario1(){
		
        Validation_Elevator_UseCase10.validate(elevatorUnderTest);
		
		Assert.assertEquals(DirectionState.DOWN, elevatorUnderTest.currentDirection());
		Assert.assertEquals(2, elevatorUnderTest.currentFloor());
		Assert.assertEquals(4, elevatorUnderTest.topFloor());
		
		for (int floor =0; floor< Validation_Elevator_UseCase10.MINIMUM_NUMBER_OF_FLOORS; floor++){
		   Assert.assertFalse(elevatorUnderTest.floorPressedInElevator(floor));
		   Assert.assertFalse(elevatorUnderTest.downPressedAtFloor(floor));
		   Assert.assertFalse(elevatorUnderTest.upPressedAtFloor(floor));
		}
	
	}

}
