package validation_tests;
import models.elevatorDoors.DoorsOnFloors;


/**
 * Validation test for {@link DoorsOnFloors}.<br>
 * The behaviour being tested is as follows:
 * <ul>
 * <li> Construct a new {@link DoorsOnFloors} test object with {@link #NUMBER_OF_FLOORS} floors </li>
 * <li> Print out the state and check the invariant </li>
 * <li> Open the doors at floor 0 </li>
 * <li> Print out the state and check the invariant </li>
 * <li> Try to open the doors at floor 1, and check that an exception is generated </li>
 * <li> Close the doors at floor 0 </li>
 * <li> Open the doors at floor 1 </li>
 * <li> Print out the state and check the invariant </li>
 * <li> Check out of bounds exceptions for open and closing doors</li>
 * </ul>
 * 
 * Expected output:
 <pre>
Constructing a new DoorsOnFloors with 10 floors
DoorState States For Elevator At Floor: 0- 1- 2- 3- 4- 5- 6- 7- 8- 9-
The invariant is true
DoorState 0 is closed
DoorState 1 is closed
DoorState 2 is closed
DoorState 3 is closed
DoorState 4 is closed
DoorState 5 is closed
DoorState 6 is closed
DoorState 7 is closed
DoorState 8 is closed
DoorState 9 is closed
Opening door 0
DoorState States For Elevator At Floor: 0-OPEN 1- 2- 3- 4- 5- 6- 7- 8- 9-
The invariant is true
DoorState 0 is open
DoorState 1 is closed
DoorState 2 is closed
DoorState 3 is closed
DoorState 4 is closed
DoorState 5 is closed
DoorState 6 is closed
DoorState 7 is closed
DoorState 8 is closed
DoorState 9 is closed
Opening door 1
Caught exception java.lang.IllegalStateException: A set of doors is already  open at another floor
Closing door 0
Opening door 1
DoorState States For Elevator At Floor: 0- 1-OPEN 2- 3- 4- 5- 6- 7- 8- 9-
The invariant is true
Checking out of bounds exceptions for open and closing doors
Opening door -1
Caught exception java.lang.IllegalArgumentException: The floor specified is not valid
Closing door -1
Caught exception java.lang.IllegalArgumentException: The floor specified is not valid
Opening door 10
Caught exception java.lang.IllegalArgumentException: The floor specified is not valid
Closing door 10
Caught exception java.lang.IllegalArgumentException: The floor specified is not valid

 </pre>
 * @see unit_tests.doors.JUnit_DoorsOnFloors
 **/
 
public class Validation_DoorsOnFloors {
	
	private final static int NUMBER_OF_FLOORS = 10;
	
	public static void main(String[] args) {
		
		System.out.println("Constructing a new DoorsOnFloors with "+NUMBER_OF_FLOORS+" floors");
		DoorsOnFloors doorsUnderTest = new DoorsOnFloors(NUMBER_OF_FLOORS);
		
		System.out.println(doorsUnderTest);
		
		System.out.println("The invariant is "+ doorsUnderTest.invariant());
		
		for (int i=0; i< NUMBER_OF_FLOORS;i++){
			System.out.print("DoorState "+i+" is ");
		    if (doorsUnderTest.isDoorOpen(i)) System.out.println ("open"); else System.out.println ("closed");
		}
		
		System.out.println("Opening door 0");
        doorsUnderTest.openDoor(0);
	    
        System.out.println(doorsUnderTest);
		
		System.out.println("The invariant is "+ doorsUnderTest.invariant());
		
		for (int i=0; i< NUMBER_OF_FLOORS;i++){
			System.out.print("DoorState "+i+" is ");
		    if (doorsUnderTest.isDoorOpen(i)) System.out.println ("open"); else System.out.println ("closed");
		}
		
		System.out.println("Opening door 1");
		try{doorsUnderTest.openDoor(1);}
		catch (IllegalStateException e){
			System.out.println("Caught exception " + e);
		}
		
		System.out.println("Closing door 0");
		doorsUnderTest.closeDoor(0);
		System.out.println("Opening door 1");
		doorsUnderTest.openDoor(1);
        System.out.println(doorsUnderTest);
		System.out.println("The invariant is "+ doorsUnderTest.invariant());
		
		System.out.println("Checking out of bounds exceptions for open and closing doors");
		
		System.out.println("Opening door -1");
		try{doorsUnderTest.openDoor(-1);}
		catch (IllegalArgumentException e){
			System.out.println("Caught exception " + e);
		}
		
		System.out.println("Closing door -1");
		try{doorsUnderTest.closeDoor(-1);}
		catch (IllegalArgumentException e){
			System.out.println("Caught exception " + e);
		}
		
		System.out.println("Opening door "+ NUMBER_OF_FLOORS);
		try{doorsUnderTest.openDoor(NUMBER_OF_FLOORS);}
		catch (IllegalArgumentException e){
			System.out.println("Caught exception " + e);
		}
		
		System.out.println("Closing door "+ NUMBER_OF_FLOORS);
		try{doorsUnderTest.closeDoor(NUMBER_OF_FLOORS);}
		catch (IllegalArgumentException e){
			System.out.println("Caught exception " + e);
		}
		

		
		System.out.println("is DoorState -1" + " open");
		try{doorsUnderTest.isDoorOpen(NUMBER_OF_FLOORS);}
		catch (IllegalArgumentException e){
			System.out.println("Caught exception " + e);
		}
		
		System.out.println("is DoorState "+ NUMBER_OF_FLOORS+ " open");
		try{doorsUnderTest.isDoorOpen(NUMBER_OF_FLOORS);}
		catch (IllegalArgumentException e){
			System.out.println("Caught exception " + e);
		}
		

	}

}
