package validation_tests.use_case1;

import interfaces.ElevatorSpecification;
import validation_tests.abstractions.Elevator_UseCase;

/**
 * A simple scenario to test the behaviour of an elevator as decided by its logic control unit.<br>
 * Initially, the elevator is on the bottom floor (0) and there is a request inside the elevator
 * to go to the top floor. <br>
 * No more requests arrive as the elevator executes its behaviour through movements decided by its controller.<br>
 * The execution stops when the elevator arrives at the top floor (or the move limit {@link #MAX_SCENARIO_LENGTH})
 * is reached.
 * @author jpaulgibson
 *
 */
public class Elevator_UseCaseScenario1 extends Elevator_UseCase{
	
	public final static int MAX_SCENARIO_LENGTH = 6;

	public Elevator_UseCaseScenario1 (ElevatorSpecification elevator) throws IllegalArgumentException {
		super(elevator);
	}

	@Override
	public int get_maximum_scenario_length() {
		
		return MAX_SCENARIO_LENGTH;
	}

	/** 
	 * There are no additional preconditions as this use case works for all elevator types and configurations,
	 * so it will always return true
	 */
	@Override
	public boolean check_elevator() {
		
		return true;
	}

	// Make a request in the elevator to go to the top floor
	@Override
	public void initialise_state() {
		ELEVATOR_UNDER_TEST.pressFloorInElevator(ELEVATOR_UNDER_TEST.topFloor());
		
	}

}
