package validation_tests.use_case10;

import java.util.Random;

import tools.DateHeader;
import tools.SeedRNGCommandLine;
import models.elevatorLogicUnit.RandomMovementLogicUnit;
import models.elevators.Elevator;
import models.elevators.ElevatorWithDoors;

/**
 * Validation test for an {@link ElevatorWithDoors} controlled by {@link RandomMovementLogicUnit}<br>
 * The use case is defined in the class {@link validation_tests.use_case10.Validation_Elevator_UseCase10}<br><br>
 * 
 * The RNG can be seeded at the command line, or a default value of 0 can be used. <br><br>
 * 
 * We use the {@link DateHeader} class to document the date/time of the test execution<br><br>
 * 
 * Typical Output (using default RNG seed = 0):
<pre>

********************************************************************
Execution Date/Time 2015/06/18 11:06:06
********************************************************************
The seed used for the random number generator in the test is 0.
You can override this value by passing an integer value as a main argument parameter, if you so wish.


Elevator State:
DirectionState = STAY
Floor = 0
Number of floors = 5
Requested Floors Inside Elevator: 1 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0-OPEN 1- 2- 3- 4-

Passing  next floor

Elevator State:
DirectionState = STAY
Floor = 0
Number of floors = 5
Requested Floors Inside Elevator: 1 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1- 2- 3- 4-

Passing  next floor

Elevator State:
DirectionState = UP
Floor = 0
Number of floors = 5
Requested Floors Inside Elevator: 1 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1- 2- 3- 4-

Stopping at next floor

Elevator State:
DirectionState = STAY
Floor = 1
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1-OPEN 2- 3- 4-

Stopping at next floor

Elevator State:
DirectionState = STAY
Floor = 1
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1-OPEN 2- 3- 4-

Passing  next floor

Elevator State:
DirectionState = UP
Floor = 1
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1- 2- 3- 4-

Stopping at next floor

Elevator State:
DirectionState = STAY
Floor = 2
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1- 2-OPEN 3- 4-

Passing  next floor

Elevator State:
DirectionState = STAY
Floor = 2
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1- 2- 3- 4-

Passing  next floor

Elevator State:
DirectionState = STAY
Floor = 2
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1- 2- 3- 4-

Stopping at next floor

Elevator State:
DirectionState = DOWN
Floor = 2
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-U 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1- 2-OPEN 3- 4-

Stopping at next floor

Elevator State:
DirectionState = STAY
Floor = 1
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-U 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1-OPEN 2- 3- 4-

Stopping at next floor

Elevator State:
DirectionState = DOWN
Floor = 1
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-U 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1-OPEN 2- 3- 4-

Stopping at next floor

Elevator State:
DirectionState = UP
Floor = 0
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-U 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0-OPEN 1- 2- 3- 4-

Passing  next floor

Elevator State:
DirectionState = STAY
Floor = 1
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-U 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1- 2- 3- 4-

Stopping at next floor

Elevator State:
DirectionState = UP
Floor = 1
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-U 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1-OPEN 2- 3- 4-

Stopping at next floor

Elevator State:
DirectionState = UP
Floor = 2
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1- 2-OPEN 3- 4-

Passing  next floor

Elevator State:
DirectionState = STAY
Floor = 3
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1- 2- 3- 4-

Stopping at next floor

Elevator State:
DirectionState = STAY
Floor = 3
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1- 2- 3-OPEN 4-

Stopping at next floor

Elevator State:
DirectionState = STAY
Floor = 3
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1- 2- 3-OPEN 4-

Stopping at next floor

Elevator State:
DirectionState = UP
Floor = 3
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1- 2- 3-OPEN 4-

Passing  next floor

Elevator State:
DirectionState = STAY
Floor = 4
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3-D 4-
Controller type = Random Controller
DoorState States For Elevator At Floor: 0- 1- 2- 3- 4-

 **** The requests were not completely serviced within the scenario length bound of 20 updates  *****

The elevator made 12 floor stops until execution stopped
</pre>
 * 
 * @author J Paul Gibson
 */
public class Validation_ElevatorWithDoorsRandom {
	
	public static void main(String[] args) {
		
		 System.out.println(DateHeader.dateString());
			
			Random rng = SeedRNGCommandLine.getRandom(args);
		
			// Make a random controller, not yet bound to (installed in) a particular elevator
			RandomMovementLogicUnit controller = new RandomMovementLogicUnit(null,rng);
			
			//Make an elevator with doors, not yet bound to a particular controller
			Elevator elevator = new ElevatorWithDoors(Validation_Elevator_UseCase10.MINIMUM_NUMBER_OF_FLOORS);

			// Install the controller to be tested in the elevator
			elevator.installLogicUnit(controller);

			// Run the validation validation_tests on the controller by binding it to an elevator
			Validation_Elevator_UseCase10.validate(elevator);
	}

}


