package validation_tests.use_case10;

import tools.DateHeader;
import models.elevatorLogicUnit.ImprovedLogicUnit;
import models.elevators.Elevator;


/**
 * Validation test for an {@link Elevator} controlled by {@link ImprovedLogicUnit}<br>
 * The use case is defined in the class {@link validation_tests.use_case10.Validation_Elevator_UseCase10}<br><br>
 * We use the {@link DateHeader} class to document the date/time of the test execution<br><br>
 * Expected output:
 <pre>
 
********************************************************************
Execution Date/Time 2015/06/08 15:12:19
********************************************************************

Elevator State:
DirectionState = STAY
Floor = 0
Number of floors = 5
Requested Floors Inside Elevator: 1 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
Controller type = Improved Stupid Controller

Stopping at next floor

Elevator State:
DirectionState = UP
Floor = 1
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-UD 3-D 4-
Controller type = Improved Stupid Controller

Stopping at next floor

Elevator State:
DirectionState = UP
Floor = 2
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3-D 4-
Controller type = Improved Stupid Controller

Passing  next floor

Elevator State:
DirectionState = UP
Floor = 3
Number of floors = 5
Requested Floors Inside Elevator: 4
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3-D 4-
Controller type = Improved Stupid Controller

Stopping at next floor

Elevator State:
DirectionState = UP
Floor = 4
Number of floors = 5
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3-D 4-
Controller type = Improved Stupid Controller

Stopping at next floor

Elevator State:
DirectionState = DOWN
Floor = 3
Number of floors = 5
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0- 1- 2-D 3- 4-
Controller type = Improved Stupid Controller

Stopping at next floor

Elevator State:
DirectionState = DOWN
Floor = 2
Number of floors = 5
Requested Floors Inside Elevator:
DirectionState Requests For Elevator At Floor: 0- 1- 2- 3- 4-
Controller type = Improved Stupid Controller

The elevator made 5 floor stops until execution stopped
</pre>
 * @author J Paul Gibson
 */
public class Validation_Elevator_ImprovedStupid  {
	
public static void main(String[] args) {
	
	 System.out.println(DateHeader.dateString());
		
		// Make an improved stupid controller, not yet bound to (installed in) a particular elevator
		ImprovedLogicUnit controller = new ImprovedLogicUnit(null);
		
		//Make an elevator without doors, not yet bound to a particular controller
		Elevator elevator = new Elevator(Validation_Elevator_UseCase10.MINIMUM_NUMBER_OF_FLOORS);

		// Install the controller to be tested in the elevator
		elevator.installLogicUnit(controller);

		// Run the validation validation_tests on the controller by binding it to an elevator
		Validation_Elevator_UseCase10.validate(elevator);
	}

}
