package validation_tests.validation_factoriesSPL;

import java.util.Random;

import interfaces.ElevatorForUsersSpecification;
import interfaces.ElevatorUserSpecification;
import enumerations.UserType;
import factoriesSPL.ElevatorFactory;
import factoriesSPL.ElevatorUserFactory;
import tools.DateHeader;
import tools.SeedRNGCommandLine;

/**
 * Validation test for {@link ElevatorUserFactory}.<br>
 * The behaviour is random (a seed value can be provided as a command line parameter):
 * <ul>
 * <li> Test default generation
 * <li> Test random generation {@link #NUMBER_OF_RANDOM_USERS} times
 * <li> Test all possible {@link enumerations.UserType#INTELLIGENT} users
 * <li> Test all possible {@link enumerations.UserType#PRESS_ALL_BUTTONS} users
 * <li> Test exceptional cases
 * </ul>
 * To set the number of tests, use the configuration variables {@link #NUMBER_OF_RANDOM_USERS}<br><br>
 * Typical output:
<pre>
The seed used for the random number generator in the test is 0.
You can override this value by passing an integer value as a main argument parameter, if you so wish.


********************************************************************
Execution Date/Time 2015/08/08 18:21:36
********************************************************************

TEST  default generation 
IntelligentElevatorUser 0 - from: 0 to: 3 currently ARRIVED_AT_FLOOR
Invariant is true

TEST  random generation 
IntelligentElevatorUser 1 - from: 3 to: 0 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 2 - from: 2 to: 1 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 3 - from: 0 to: 2 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 4 - from: 1 to: 2 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 5 - from: 3 to: 0 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 6 - from: 0 to: 1 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUser 7 - from: 0 to: 1 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 8 - from: 2 to: 0 currently ARRIVED_AT_FLOOR respects the invariant
PressAllButtonsElevatorUser 9 - from: 2 to: 0 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 10 - from: 2 to: 0 currently ARRIVED_AT_FLOOR respects the invariant

TEST  explicit generation  of all possible INTELLIGENT users : 
IntelligentElevatorUser 11 - from: 0 to: 1 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUser 12 - from: 0 to: 2 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUser 13 - from: 0 to: 3 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUser 14 - from: 1 to: 0 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUser 15 - from: 1 to: 2 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUser 16 - from: 1 to: 3 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUser 17 - from: 2 to: 0 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUser 18 - from: 2 to: 1 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUser 19 - from: 2 to: 3 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUser 20 - from: 3 to: 0 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUser 21 - from: 3 to: 1 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUser 22 - from: 3 to: 2 currently ARRIVED_AT_FLOOR respects the invariant

TEST  explicit generation  of all possible PRESS_ALL_BUTTONS users : 
PressAllButtonsElevatorUser 23 - from: 0 to: 1 currently ARRIVED_AT_FLOOR respects the invariant
PressAllButtonsElevatorUser 24 - from: 0 to: 2 currently ARRIVED_AT_FLOOR respects the invariant
PressAllButtonsElevatorUser 25 - from: 0 to: 3 currently ARRIVED_AT_FLOOR respects the invariant
PressAllButtonsElevatorUser 26 - from: 1 to: 0 currently ARRIVED_AT_FLOOR respects the invariant
PressAllButtonsElevatorUser 27 - from: 1 to: 2 currently ARRIVED_AT_FLOOR respects the invariant
PressAllButtonsElevatorUser 28 - from: 1 to: 3 currently ARRIVED_AT_FLOOR respects the invariant
PressAllButtonsElevatorUser 29 - from: 2 to: 0 currently ARRIVED_AT_FLOOR respects the invariant
PressAllButtonsElevatorUser 30 - from: 2 to: 1 currently ARRIVED_AT_FLOOR respects the invariant
PressAllButtonsElevatorUser 31 - from: 2 to: 3 currently ARRIVED_AT_FLOOR respects the invariant
PressAllButtonsElevatorUser 32 - from: 3 to: 0 currently ARRIVED_AT_FLOOR respects the invariant
PressAllButtonsElevatorUser 33 - from: 3 to: 1 currently ARRIVED_AT_FLOOR respects the invariant
PressAllButtonsElevatorUser 34 - from: 3 to: 2 currently ARRIVED_AT_FLOOR respects the invariant

TEST  explicit generation  of all possible INTELLIGENT_WITH_KEY users : 
IntelligentElevatorUserWithKey 35 - from: 0 to: 1 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 36 - from: 0 to: 2 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 37 - from: 0 to: 3 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 38 - from: 1 to: 0 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 39 - from: 1 to: 2 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 40 - from: 1 to: 3 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 41 - from: 2 to: 0 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 42 - from: 2 to: 1 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 43 - from: 2 to: 3 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 44 - from: 3 to: 0 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 45 - from: 3 to: 1 currently ARRIVED_AT_FLOOR respects the invariant
IntelligentElevatorUserWithKey 46 - from: 3 to: 2 currently ARRIVED_AT_FLOOR respects the invariant

Test exceptional cases
Correctly caught exception java.lang.IllegalArgumentException: You cannot randomize using a null random number generator
Correctly caught exception java.lang.IllegalArgumentException: You cannot assign a user to a null elevator
Correctly caught exception java.lang.IllegalArgumentException: start floor is out of bounds
Correctly caught exception java.lang.IllegalArgumentException: end floor is out of bounds
Correctly caught exception java.lang.IllegalArgumentException: start and end floors must be different
Correctly caught exception java.lang.IllegalArgumentException: You cannot assign a user to a null elevator
Correctly caught exception java.lang.IllegalArgumentException: You cannot assign a user to a null elevator
Correctly caught exception java.lang.IllegalArgumentException: You cannot randomize using a null random number generator
</pre>
 * @author jpaulgibson
 * @see unit_tests.factoriesSPL.JUnit_ElevatorUserFactory
 **/
public class Validation_ElevatorUserFactory {
	
	 /**
     * The number of test generations of random users
     */
	 final static int NUMBER_OF_RANDOM_USERS =10;
	 
	 
	 final static ElevatorUserFactory factoryUnderTest = ElevatorUserFactory.getInstance();

	 /**
	  * The default elevator is constructed in order to test the user behaviour
	  */
	 final static ElevatorForUsersSpecification ELEVATOR = 
				ElevatorFactory.makeElevator();
	 
	
	public static void main(String[] args) {
		
		 
		final Random RNG = SeedRNGCommandLine.getRandom(args);
			System.out.println(DateHeader.dateString());
			
	    ElevatorUserSpecification user;
	     
	    // Test default generation
	    user = factoryUnderTest.makeElevatorUser();
	    System.out.println("\nTEST  default generation ");
	    System.out.println(user );
	    System.out.println("Invariant is "+user.invariant());
	
				
		// Test random generation
	    System.out.println("\nTEST  random generation ");
	     for (int testCount = 0; testCount<NUMBER_OF_RANDOM_USERS; testCount++){
				 user = factoryUnderTest.makeElevatorUser(RNG, ELEVATOR);
				 System.out.print(user);
				 if (user.invariant())  System.out.println(" respects the invariant");
				                    else System.out.println(" does not respect the invariant");}
	     
	     // Test all possible configurations
	     
	     for (UserType user_type: UserType.values()){
		 System.out.println("\nTEST  explicit generation  of all possible " + user_type + " users : ");

		     for (int from = 0; from <= ELEVATOR.topFloor(); from++)
					for (int to = 0; to <= ELEVATOR.topFloor(); to++)
						if (from!=to){
				            user = factoryUnderTest.makeElevatorUser(user_type, from, to, ELEVATOR);
				            System.out.print(user);
				            if (user.invariant())  System.out.println(" respects the invariant");
		                    else System.out.println(" does not respect the invariant");}
		 }
		 
		 //Test exceptional cases
		 System.out.println("\nTest exceptional cases");
		 
		 try{ user = factoryUnderTest.makeElevatorUser(null, ELEVATOR);}
		  catch (IllegalArgumentException e){
			  System.out.println("Correctly caught exception "+e);
		  }
		 
		 try{ user = factoryUnderTest.makeElevatorUser(RNG, null);}
		  catch (IllegalArgumentException e){
			  System.out.println("Correctly caught exception "+e);
		  }
		 
		 try{ user = factoryUnderTest.makeElevatorUser(UserType.INTELLIGENT, -1, 0, ELEVATOR);}
		  catch (IllegalArgumentException e){
			  System.out.println("Correctly caught exception "+e);
		  }
		 
		 try{ user = factoryUnderTest.makeElevatorUser(UserType.INTELLIGENT, 0, -1, ELEVATOR);}
		  catch (IllegalArgumentException e){
			  System.out.println("Correctly caught exception "+e);
		  }
		 
		 try{ user = factoryUnderTest.makeElevatorUser(UserType.INTELLIGENT, 1, 1, ELEVATOR);}
		  catch (IllegalArgumentException e){
			  System.out.println("Correctly caught exception "+e);
		  }
		 
		 try{ user = factoryUnderTest.makeElevatorUser(UserType.INTELLIGENT, 1, 1, null);}
		  catch (IllegalArgumentException e){
			  System.out.println("Correctly caught exception "+e);
		  }
		 
		 try{ user = factoryUnderTest.makeElevatorUser(UserType.PRESS_ALL_BUTTONS, RNG, null);}
		  catch (IllegalArgumentException e){
			  System.out.println("Correctly caught exception "+e);
		  }
		 
		 try{ user = factoryUnderTest.makeElevatorUser(UserType.PRESS_ALL_BUTTONS, null, ELEVATOR);}
		  catch (IllegalArgumentException e){
			  System.out.println("Correctly caught exception "+e);
		  }
	}

}
