package validation_tests.validation_factoriesSPL;

import interfaces.LogicUnitForElevatorSpecification;

import java.util.Random;

import tools.DateHeader;
import tools.SeedRNGCommandLine;
import enumerations.LogicType;
import factoriesSPL.LogicUnitFactory;

/**
 * Validation test for {@link LogicUnitFactory}.<br>
 * The behaviour is random (a seed value can be provided as a command line parameter):
 * <ul>
 * <li> Validation test for {@link LogicUnitFactory#makeLogicUnit(Random ) }<br>
 *      Tests that the factory builds randomized controllers  <br>
 *      Does this by running the random constructor {@link #NUMBER_OF_RANDOM_CONTROLLERS} times.</li>
 * <li> Validation test for {@link LogicUnitFactory#makeLogicUnit(LogicType ) }<br>
 *      Tests that the factory can build all possible controller types as defined in
 *       {@link enumerations.LogicType}.</li>
 * </ul>
 * To set the number of tests, use the configuration variables {@link #NUMBER_OF_RANDOM_CONTROLLERS} <br><br>
 * Typical output:
<pre>
The seed used for the random number generator in the test is 0.
You can override this value by passing an integer value as a main argument parameter, if you so wish.


********************************************************************
Execution Date/Time 2015/07/28 11:49:10
********************************************************************

TEST  random generation 0: Random Controller
TEST  random generation 1: Emergency Controller
TEST  random generation 2: Smart Controller
TEST  random generation 3: Improved Stupid Controller
TEST  random generation 4: Random Controller
TEST  random generation 5: Emergency Controller
TEST  random generation 6: Stupid Controller
TEST  random generation 7: Stupid Controller
TEST  random generation 8: Smart Controller
TEST  random generation 9: Smart Controller
TEST  explicit generation of : Random Controller
TEST  explicit generation of : Stupid Controller
TEST  explicit generation of : Improved Stupid Controller
TEST  explicit generation of : Emergency Controller
TEST  explicit generation of : Smart Controller
</pre>
 * @author jpaulgibson
 * @see unit_tests.factoriesSPL.JUnit_LogicUnitFactory
 **/
public class Validation_LogicUnitFactory {
	
	 final static int NUMBER_OF_RANDOM_CONTROLLERS =10;
	 
	 /**
	  * @param args may be used to seed the random number generator for the validation tests
	  */
	 public static void main(String[] args) {
			
		 LogicUnitForElevatorSpecification controller = null;
		 
			final Random RNG = SeedRNGCommandLine.getRandom(args);
			System.out.println(DateHeader.dateString());
				
			/*
			 * Validation test for {@link LogicUnitFactory#makeController(Random ) }<br>
			 * Tests that the factory builds randomized controllers  <br>
			 * Does this by running the random constructor a specified number of times.
			 */
			 for (int testCount = 0; testCount<NUMBER_OF_RANDOM_CONTROLLERS; testCount++){
			       System.out.print("\nTEST  random generation "+ testCount+": ");
				   controller = LogicUnitFactory.makeLogicUnit(RNG);
				   System.out.print(controller);
			 }
			 
			 /*
		      * Validation test for {@link LogicUnitFactory#makeController(LogicType ) }<br>
			  * Tests that the factory can build all possible controller types  <br>
		      */
			
			 int NUMBER_OF_CONTROLLER_TYPES = LogicType.values().length;
			 for (int controller_index =0; controller_index < NUMBER_OF_CONTROLLER_TYPES; controller_index++){
				 System.out.print("\nTEST  explicit generation of : ");
				 controller = LogicUnitFactory.makeLogicUnit(LogicType.values()[controller_index]);
			   System.out.print(controller);
			 }
	 }
}
